######################################################################
# Author: Michael Hhle <hoehle@stat.uni-muenchen.de
# Date:   4 March 2003
######################################################################


######################################################################
# Calculate R0 according to Becker89 Equation 7.4.3 possibly
# containing correction. This is a vectorized version also
# able to handle several experiments as in DeJong.
#
# Parameters:
#  S0  - Number of susceptibles at time 0
#  STR - Number of susceptibles at time T_N
#  RTN - R(T_N) number of recovered at time T_N
#  N   - Population size.
#  fbe - Fraction of \int_0^{T_N} I(t) dt spent BEFORE T_R (S(t)=0)
#        I.e 1-(wasted infectious periods)
######################################################################

r0.hat <- function(S0,STR,RTN,N,fbe=1) {
  # sum(1/(S0:(STR+1)))    / (fbe*RTN) 
  #New vectorized version..well..we loop, coz no smart way to make the
  #uneven long sequences in R?
  top <- 0
  bottom <- 0
  for (i in 1:length(S0)) {
    top <-  top + sum(1/(S0[i]:(STR[i]+1)))
    bottom <- bottom + fbe[i]*RTN[i]
  }
  return(N*top/bottom)
}

######################################################################
# Calculate Standard Deviation of r0 estimate
#
# Parameters:
#  S0  - Number of susceptibles at time 0
#  STR - Number of susceptibles at time T_N
#  RTN - R(T_N) number of recovered at time T_N
#  N   - Population size. (not a vector!)
#  fbe - Fraction of \int_0^{T_N} I(t) dt spent BEFORE T_R (S(t)=0)
#        I.e the wasted infectious periods
######################################################################

r0.se <- function(S0,STR,RTN,N,fbe=1) {
  #Account variables
  top <- 0
  bottom <- 0

  #Calculate theta. Luckily we only accept common N.
  theta <- r0.hat(S0,STR,RTN,N,fbe)/N

  for (i in 1:length(S0)) {
    #Correct number of recoveries at time TN
    R <- fbe[i]*RTN[i]
    
    #Find its s.e.
    top <- top + sum(1/(S0[i]:(STR[i]+1))^2) + theta^2*R
    bottom <- bottom + R
  }
  
  #Result. 
  return(N*sqrt(top)/bottom)
}

######################################################################
# Estimate R0
#
# Parameters:
#  S0  - Number of susceptibles at time 0
#  STR - Number of susceptibles at time T_N
#  RTN - R(T_N) number of recovered at time T_N
#  N   - Population size.
#  fbe - Fraction of \int_0^{T_N} I(t) dt spent BEFORE T_R (S(t)=0)
#        I.e the wasted infectious periods
######################################################################

r0 <- function(S0,STR,RTN,N,fbe=1) {
  res <- c(r0.hat(S0,STR,RTN,N,fbe),r0.se(S0,STR,RTN,N,fbe))
  names(res) <- c("r0","s.e.(r0)")
  return(res)
}
  
######################################################################
# ESS function
######################################################################

doIt <- function() {
  source("R/dejong.R")

  #Data from deJong article.
  I <- read.table("R/dejong.txt",header=T)
  #Experiment 1 a
  I1 <- I[I$exp==1,]
  I2 <- I[I$exp==2,]
  #Find fraction of infection occuring before TN
  f1 <- 1-sum(I1[I1$t>=6,]$nonvacc)/sum(I1$nonvacc) #TN = 6
  f2 <- 1-sum(I2[I2$t>=3,]$nonvacc)/sum(I2$nonvacc) #TN = 3

  #p.10 Non vaccinated 
  r0(c(5,5),c(0,0),c(10,10),10,c(f1,f2))

  #p.10: Vaccinated population. Experiment 1. Initially 5 inf and 5
  #susceptibles. At stop we have still 3 susceptibles and 7 recovered.
  r0(5,3,7,10,1)
  r0(5,4,5,9,1)

  #The experiment in Laevens
  #S0,STN, R(TN), Frac
  r0(5,0,6,6,1)

  #Pen 2 in Dewulf
  r01 <- 15*0.09859152749114948/0.06976742462929769
  n <- 14
  m <- 1
  Z <- 14/n #S(T_N)/n
  STN <- 14
  mu <- 1/n #I(0) = \mu * n, where n=14 in our case.
  #se.r01 <- r01*sqrt( (1/Z+1/(Z+mu))/(n+m) )
  se.r01 <- r01*sqrt( 1/STN + 1/(STN+m) )

  #Read data from Dewulf Transmission experiment - see
  #the Hoehle, Joergensen, O'Neill applied statistics
  #paper for details.
  pen2 <-
    read.table("dewulf/csfexp-full-unit2.data",,skip=6,
               col.names=c("x","y","E","I","R","D"))
  
  #Find TN - the time, where there are no more susceptibles left.
  TN <- max(pen2$E)
  #Find fraction of infection occuring before TN. I.e. sum inf. period
  #between 0 and TN for each individual and divide by total
  #inf. period
  leftOfTN <- pmin(pen2$R,TN)-pen2$I
  f <- sum((leftOfTN>0)*leftOfTN)/sum(pen2$R-pen2$I)

  #Calculate R0
  r0(14,0,15,15,f)
  
}







